// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;
import java.net.InetAddress;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * Address Record - maps a domain name to an Internet address
 * 
 * @author Brian Wellington
 */

public class AAAARecord extends Record {

	private InetAddress address;

	/**
	 * Creates an A Record from the given data
	 * 
	 * @param address The address that the name refers to
	 */
	public AAAARecord(Name _name, short _dclass, int _ttl, InetAddress _address) throws IOException {
		super(_name, Type.AAAA, _dclass, _ttl);
		address = _address;
	}

	AAAARecord() {
	}

	AAAARecord(Name _name, short _dclass, int _ttl, int length, DataByteInputStream in, Compression c) throws IOException {
		super(_name, Type.AAAA, _dclass, _ttl);

		if (in == null) return;

		final byte[] data = new byte[16];
		in.read(data);
		
		if (data.length > 16)
			throw new IOException("An Inet6Address is only 128 bits");

		address = InetAddress.getByName(byteAddressToString(data));
	}

	AAAARecord(Name _name, short _dclass, int _ttl, MyStringTokenizer st, Name origin) throws IOException {
		super(_name, Type.AAAA, _dclass, _ttl);
		final String s = st.nextToken();
		if (s.equals("@me@")) {
			address = InetAddress.getLocalHost();
			if (address.equals(InetAddress.getByName("127.0.0.1"))) {
				throw new IOException("InetAddress.getLocalHost() is broken.");
			}
		} else {
			address = InetAddress.getByName(s);
		}
	}

	/** Returns the Internet address */
	public InetAddress getAddress() {
		return address;
	}

	/** Converts to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (address != null) {
			sb.append(address.getHostAddress());
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (address == null) return;

		out.write(address.getAddress());
	}
	
	private String byteAddressToString(byte[] data) throws IOException {
		if (data.length > 16)
			throw new IOException("An Inet6Address is only 128 bits");
		int [] labels = new int[8];
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < 8; i++)
			labels[i] = ((data[2 * i] & 0xFF) << 8) +
				    (data[2 * i + 1] & 0xFF);
		int start = -1, length = -1, tstart = -1, tlength = -1;
		boolean inzero = false;
		for (int i = 0; i < 8; i++) {
			if (!inzero) {
				if (labels[i] == 0) {
					tstart = i;
					tlength = 0;
					inzero = true;
				}
			}
			else {
				if (labels[i] == 0)
					tlength++;
				else {
					inzero = false;
					if (tlength > length) {
						start = tstart;
						length = tlength;
					}
					tlength = -1;
				}
			}
		}
		if (tlength > length) {
			start = tstart;
			length = tlength;
		}

		if (labels[0] == 0 && labels[1] == 0 && labels[2] == 0 &&
		    labels[3] == 0 && labels[4] == 0 && labels[6] != 0 &&
		    (labels[5] == 0 || labels[5] == 0xFFFF))
		{
			sb.append("::");
			if (labels[5] == 0xFFFF)
				sb.append("FFFF:");
			sb.append(labels[6] >> 8);
			sb.append(".");
			sb.append(labels[6] & 0xFF);
			sb.append(".");
			sb.append(labels[7] >> 8);
			sb.append(".");
			sb.append(labels[7] & 0xFF);
		}
		else if (start == -1) {
			for (int i = 0; i < 8; i++) {
				sb.append(Integer.toHexString(labels[i]).toUpperCase());
				if (i != 7)
					sb.append(":");
			}
		}
		else {
			for (int i = 0; i < start; i++) {
				sb.append(Integer.toHexString(labels[i]).toUpperCase());
				sb.append(":");
			}
			if (start == 0)
				sb.append(":");
			sb.append(":");
			for (int i = start + length + 1; i < 8 ; i++) {
				sb.append(Integer.toHexString(labels[i]).toUpperCase());
				if (i != 7)
					sb.append(":");
			}
		}
		return sb.toString();
	}

}
