// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * A helper class that tries to locate name servers and the search path to be
 * appended to unqualified names. Currently, this works if either the
 * appropriate properties are set, or the OS has a unix-like /etc/resolv.conf.
 * There is no reason for these routines to be called directly except curiosity.
 * 
 * @author Brian Wellington
 */

public class FindServer {

	private static String[] server = null;
	private static Name[] search = null;
	private static boolean probed = false;

	private FindServer() {
	}

	/** Returns all entries in the located search path */
	public static Name[] searchPath() {
		probe();
		return search;
	}

	/** Returns the first located server */
	public static String server() {
		final String[] array = servers();
		if (array == null)
			return null;
		else
			return array[0];
	}

	/** Returns all located servers */
	public static String[] servers() {
		probe();
		return server;
	}

	/**
	 * Looks in the system properties to find servers and a search path.
	 * Properties of the form dns.server1, dns.server2, etc. define servers.
	 * Properties of the form dns.search1, dns.seearch, etc. define the search
	 * path.
	 */
	private static void findProperty() {
		String s;
		Vector<String> v = null;
		for (int i = 1; i <= 5; i++) {
			s = System.getProperty("dns.server" + i);
			if (s == null) {
				break;
			}
			if (v == null) {
				v = new Vector<String>();
			}
			v.addElement(s);
		}
		if (v != null) {
			server = new String[v.size()];
			for (int i = 0; i < v.size(); i++) {
				server[i] = v.elementAt(i);
			}
		}

		v = null;
		for (int i = 1; i <= 5; i++) {
			s = System.getProperty("dns.search" + i);
			if (s == null) {
				break;
			}
			if (v == null) {
				v = new Vector<String>();
			}
			v.addElement(s);
		}
		if (v != null) {
			search = new Name[v.size()];
			for (int i = 0; i < v.size(); i++) {
				search[i] = new Name(v.elementAt(i));
			}
		}
	}

	/**
	 * Looks in /etc/resolv.conf to find servers and a search path. "nameserver"
	 * lines specify servers. "domain" and "search" lines define the search
	 * path.
	 */
	private static void findUnix() {
		InputStream in = null;
		try {
			in = new FileInputStream("/etc/resolv.conf");
		} catch (final FileNotFoundException e) {
			return;
		}
		final InputStreamReader isr = new InputStreamReader(in);
		final BufferedReader br = new BufferedReader(isr);
		Vector<String> vserver = null;
		Vector<String> vsearch = null;
		try {
			String line;
			while ((line = br.readLine()) != null) {
				if (line.startsWith("nameserver")) {
					if (vserver == null) {
						vserver = new Vector<String>();
					}
					final StringTokenizer st = new StringTokenizer(line);
					st.nextToken(); /* skip nameserver */
					vserver.addElement(st.nextToken());
				} else if (line.startsWith("domain")) {
					if (vsearch == null) {
						vsearch = new Vector<String>();
					}
					final StringTokenizer st = new StringTokenizer(line);
					st.nextToken(); /* skip domain */
					final String s = st.nextToken();
					if (!vsearch.contains(s)) {
						vsearch.addElement(s);
					}
				} else if (line.startsWith("search")) {
					if (vsearch == null) {
						vsearch = new Vector<String>();
					}
					final StringTokenizer st = new StringTokenizer(line);
					st.nextToken(); /* skip domain */
					String s;
					while (st.hasMoreTokens()) {
						s = st.nextToken();
						if (!vsearch.contains(s)) {
							vsearch.addElement(s);
						}
					}
				}
			}
		} catch (final IOException e) {
		}
		if (server == null && vserver != null) {
			server = new String[vserver.size()];
			for (int i = 0; i < vserver.size(); i++) {
				server[i] = vserver.elementAt(i);
			}
		}
		if (search == null && vsearch != null) {
			search = new Name[vsearch.size()];
			for (int i = 0; i < vsearch.size(); i++) {
				search[i] = new Name(vsearch.elementAt(i));
			}
		}
	}

	private static void probe() {
		if (probed) return;
		probed = true;
		findProperty();
		if (server != null && search != null) return;
		findUnix();
		return;
	}

}
