// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;
import java.util.Random;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;

/**
 * A DNS message header
 * 
 * @see Message
 * 
 * @author Brian Wellington
 */

public class Header {

	private int id;
	private boolean[] flags;
	private byte rcode, opcode;
	private short[] counts;

	/**
	 * Create a new empty header with a random message id
	 */
	public Header() {
		this(randomID());
	}

	/**
	 * Parses a Header from a stream containing DNS wire format. This normally
	 * isn't useful to clients.
	 */
	public Header(DataByteInputStream in) throws IOException {
		this(in.readUnsignedShort());
		readFlags(in);
		for (int i = 0; i < counts.length; i++) {
			counts[i] = in.readShort();
		}
	}

	/**
	 * Create a new empty header.
	 * 
	 * @param id The message id
	 */
	public Header(int _id) {
		counts = new short[4];
		flags = new boolean[16];
		id = _id;
	}

	/* Creates a new Header identical to the current one */
	@Override
	public Object clone() {
		final Header h = new Header();
		for (int i = 0; i < counts.length; i++) {
			h.counts[i] = counts[i];
		}
		for (int i = 0; i < flags.length; i++) {
			h.flags[i] = flags[i];
		}
		h.id = id;
		h.rcode = rcode;
		h.rcode = rcode;
		h.opcode = opcode;
		return h;
	}

	/**
	 * Retrieves the record count for the given section
	 * 
	 * @see Section
	 */
	public short getCount(int field) {
		return counts[field];
	}

	/**
	 * Retrieves a flag
	 * 
	 * @see Flags
	 */
	public boolean getFlag(int bit) {
		return flags[bit];
	}

	/**
	 * Retrieves the message ID
	 */
	public int getID() {
		return id & 0xFFFF;
	}

	/**
	 * Retrieves the mesasge's opcode
	 * 
	 * @see Opcode
	 */
	public byte getOpcode() {
		return opcode;
	}

	/**
	 * Retrieves the mesasge's rcode
	 * 
	 * @see Rcode
	 */
	public byte getRcode() {
		return rcode;
	}

	/** Converts the header's flags into a String */
	public String printFlags() {
		String s;
		final StringBuffer sb = new StringBuffer();

		for (int i = 0; i < flags.length; i++)
			if (getFlag(i) && (s = Flags.string(i)) != null) {
				sb.append(s);
				sb.append(" ");
			}
		return sb.toString();
	}

	/**
	 * Sets a flag to the supplied value
	 * 
	 * @see Flags
	 */
	public void setFlag(int bit) {
		flags[bit] = true;
	}

	/**
	 * Sets the message ID
	 */
	public void setID(int _id) {
		id = _id;
	}

	/**
	 * Sets the message's opcode
	 * 
	 * @see Opcode
	 */
	public void setOpcode(byte value) {
		opcode = value;
	}

	/**
	 * Sets the message's rcode
	 * 
	 * @see Rcode
	 */
	public void setRcode(byte value) {
		rcode = value;
	}

	/** Converts the header into a String */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();

		sb.append(";; ->>HEADER<<- ");
		sb.append("opcode: " + Opcode.string(getOpcode()));
		sb.append(", status: " + Rcode.string(getRcode()));
		sb.append(", id: " + getID());
		sb.append("\n");

		sb.append(";; flags: " + printFlags());
		sb.append("; ");
		for (int i = 0; i < 4; i++) {
			sb.append(Section.string(i) + ": " + getCount(i) + " ");
		}
		return sb.toString();
	}

	public byte[] toWire() throws IOException {
		final DataByteOutputStream out = new DataByteOutputStream();
		toWire(out);
		return out.toByteArray();
	}

	/**
	 * Sets a flag to the supplied value
	 * 
	 * @see Flags
	 */
	public void unsetFlag(int bit) {
		flags[bit] = false;
	}

	private void readFlags(DataByteInputStream in) throws IOException {
		final short flags1 = (short) in.readUnsignedByte();
		final short flags2 = (short) in.readUnsignedByte();
		for (int i = 0; i < 8; i++) {
			flags[i] = (flags1 & 1 << 7 - i) != 0;
			flags[i + 8] = (flags2 & 1 << 7 - i) != 0;
		}
		opcode = (byte) (flags1 >> 3 & 0xF);
		rcode = (byte) (flags2 & 0xF);
	}

	private void writeFlags(DataByteOutputStream out) throws IOException {
		short flags1 = 0, flags2 = 0;
		for (int i = 0; i < 8; i++) {
			if (flags[i]) {
				flags1 |= 1 << 7 - i;
			}
			if (flags[i + 8]) {
				flags2 |= 1 << 7 - i;
			}
		}
		flags1 |= opcode << 3;
		flags2 |= rcode;
		out.writeByte(flags1);
		out.writeByte(flags2);
	}

	void decCount(int field) {
		counts[field]--;
	}

	boolean[] getFlags() {
		return flags;
	}

	void incCount(int field) {
		counts[field]++;
	}

	void setCount(int field, int value) {
		counts[field] = (short) value;
	}

	void toWire(DataByteOutputStream out) throws IOException {
		out.writeShort(getID());
		writeFlags(out);
		for (final short count : counts) {
			out.writeShort(count);
		}
	}

	/**
	 * Generates a random number suitable for use as a message ID
	 */
	static short randomID() {
		final Random random = new Random();
		return (short) (random.nextInt() & 0xFFFF);
	}

}
