// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;

/**
 * Options - describes extended DNS (EDNS) properties of a Message. No specific
 * options are defined other than those specified in the header. An OPT should
 * be generated by Resolver.
 * 
 * @see EDNS
 * @see Message
 * @see Resolver
 * 
 * @author Brian Wellington
 */

public class OPTRecord extends Record {

	private Hashtable<Integer, byte[]> options;

	/**
	 * Creates an OPT Record with no data. This is normally called by the EDNS
	 * class
	 * 
	 * @see EDNS
	 */
	public OPTRecord(Name _name, short _dclass, int _ttl) {
		super(_name, Type.OPT, _dclass, _ttl);
		options = null;
	}

	OPTRecord() {
	}

	OPTRecord(Name _name, short _dclass, int _ttl, int length, DataByteInputStream in, Compression c)
			throws IOException {
		super(_name, Type.OPT, _dclass, _ttl);
		if (in == null) return;
		int count = 0;
		if (count < length) {
			options = new Hashtable<Integer, byte[]>();
		}
		while (count < length) {
			final int code = in.readUnsignedShort();
			final int len = in.readUnsignedShort();
			final byte[] data = new byte[len];
			in.read(data);
			count += 4 + len;
			options.put(new Integer(code), data);
		}
	}

	/**
	 * Returns the extended Rcode
	 * 
	 * @see Rcode
	 */
	public short getExtendedRcode() {
		return (short) (ttl >>> 24);
	}

	/** Returns the maximum allowed payload size. */
	public short getPayloadSize() {
		return dclass;
	}

	/** Returns the highest support EDNS version */
	public short getVersion() {
		return (short) (ttl >>> 16 & 0xFF);
	}

	/** Converts to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (options != null) {
			final Enumeration<Integer> e = options.keys();
			while (e.hasMoreElements()) {
				final Integer i = e.nextElement();
				sb.append(i + " ");
			}
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (options == null) return;
		final Enumeration<Integer> e = options.keys();
		while (e.hasMoreElements()) {
			final Integer i = e.nextElement();
			final short key = i.shortValue();
			out.writeShort(key);
			final byte[] data = options.get(i);
			out.writeShort(data.length);
			out.write(data);
		}
	}

	@Override
	void rrToWireCanonical(DataByteOutputStream out) throws IOException {
		throw new IOException("An OPT should never be converted to canonical");
	}

}
