// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

/**
 * A set of Records with the same name, type, and class. Also included are all
 * SIG records signing the data records.
 * 
 * @see Record
 * @see SIGRecord
 * 
 * @author Brian Wellington
 */

public class RRset {

	private Vector<Record> rrs;

	private Vector<Record> sigs;
	private int start;

	/** Creates an empty RRset */
	public RRset() {
		rrs = new Vector<Record>();
		sigs = new Vector<Record>();
		start = 0;
	}

	/** Adds a Record to an RRset */
	public void addRR(Record r) {
		if (r.getType() != Type.SIG) {
			if (!rrs.contains(r)) {
				rrs.addElement(r);
			}
		} else {
			if (!sigs.contains(r)) {
				sigs.addElement(r);
			}
		}
	}

	/** Deletes all Records from an RRset */
	public void clear() {
		rrs.setSize(0);
		sigs.setSize(0);
		start = 0;
	}

	/** Returns the first record */
	public Record first() {
		if (rrs.size() == 0) return null;
		return rrs.elementAt(0);
	}

	/**
	 * Returns the class of the records see @DClass
	 */
	public short getDClass() {
		if (rrs.size() == 0) return 0;
		final Record r = rrs.elementAt(0);
		return r.getDClass();
	}

	/**
	 * Returns the name of the records see @Name
	 */
	public Name getName() {
		if (rrs.size() == 0) return null;
		final Record r = rrs.elementAt(0);
		return r.getName();
	}

	/** Returns the ttl of the records */
	public int getTTL() {
		if (rrs.size() == 0) return 0;
		int ttl = Integer.MAX_VALUE;
		final Enumeration<Record> e = rrs();
		while (e.hasMoreElements()) {
			final Record r = e.nextElement();
			if (r.getTTL() < ttl) {
				ttl = r.getTTL();
			}
		}
		return ttl;
	}

	/**
	 * Returns the type of the records see @Type
	 */
	public short getType() {
		if (rrs.size() == 0) return 0;
		final Record r = rrs.elementAt(0);
		return r.getType();
	}

	/**
	 * Returns an Enumeration listing all (data) records. This cycles through
	 * the records, so each Enumeration will start with a different record.
	 */
	public Enumeration<Record> rrs() {
		return new Enumerator();
	}

	/** Returns an Enumeration listing all signature records */
	public Enumeration<Record> sigs() {
		return sigs.elements();
	}

	/** Returns the number of (data) records */
	public int size() {
		return rrs.size();
	}

	/** Converts the RRset to a String */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("{ [");
		Enumeration<Record> e = rrs();
		while (e.hasMoreElements()) {
			final Record rr = e.nextElement();
			sb.append(rr);
			if (e.hasMoreElements()) {
				sb.append("<>");
			}
		}
		sb.append("] [");
		e = sigs();
		while (e.hasMoreElements()) {
			final Record rr = e.nextElement();
			sb.append(rr);
			if (e.hasMoreElements()) {
				sb.append("<>");
			}
		}
		sb.append("] }");
		return sb.toString();
	}

	class Enumerator implements Enumeration<Record> {
		int first, count, size;
		Record[] records;
		boolean cycled;

		Enumerator() {
			size = rrs.size();
			records = new Record[size];
			for (int i = 0; i < size; i++) {
				records[i] = rrs.elementAt(i);
			}
			if (++start >= size) {
				start -= size;
			}
			first = count = start;
		}

		@Override
		public boolean hasMoreElements() {
			return !cycled || count < first;
		}

		@Override
		public Record nextElement() {
			if (count == first && cycled) throw new NoSuchElementException();
			final Object o = records[count++];
			if (count == size) {
				count = 0;
				cycled = true;
			}
			return (Record) o;
		}
	}

}
